// config_report.cc. Generated from config.report by tools/mk_cfgrpt.sh

namespace isc {
namespace detail {

extern const char* const config_report[] = {
    ";;;;        Kea source configure results:",
    ";;;;     -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-",
    ";;;; ",
    ";;;; Package:",
    ";;;;   Name:               kea",
    ";;;;   Version:            3.2.0-git",
    ";;;;   Extended version:   3.2.0-git (git e1855213f333e96f65b952676cc03f93942b1e57)",
    ";;;;   Version type:       stable",
    ";;;;   OS Family:          Linux",
    ";;;; ",
    ";;;;   Prefix:             /usr",
    ";;;;   Hooks directory:    /usr/lib/kea/hooks",
    ";;;;   Premium hooks:      no",
    ";;;; ",
    ";;;; Meson Version:        1.11.1",
    ";;;; Build Options:        -Dbuildtype=plain -Dprefix=/usr -Dlibexecdir=lib -Dsbindir=bin -Dauto_features=enabled -Dwrap_mode=nodownload -Db_pie=true -Dpython.bytecompile=1 -Dnetconf=disabled -Dtests=enabled -Dinstall_umask=0022 -Drunstatedir=/run",
    ";;;; ",
    ";;;; C++ Compiler:",
    ";;;;   CXX:                c++",
    ";;;;   CXX_ID:             gcc",
    ";;;;   CXX_VERSION:        c++ (GCC) 16.1.1 20260625",
    ";;;;   CXX_STANDARD:       202002L",
    ";;;;   CXX_ARGS:           -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security -fstack-clash-protection -fcf-protection -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -Wp,-D_GLIBCXX_ASSERTIONS -g -ffile-prefix-map=/build/kea/src=/usr/src/debug/kea -flto=auto",
    ";;;;   LD_ID:              ld.bfd",
    ";;;;   LD_ARGS:            -Wl,-O1 -Wl,--sort-common -Wl,--as-needed -Wl,-z,relro -Wl,-z,now -Wl,-z,pack-relative-relocs -flto=auto -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security -fstack-clash-protection -fcf-protection -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -Wp,-D_GLIBCXX_ASSERTIONS -g -ffile-prefix-map=/build/kea/src=/usr/src/debug/kea -flto=auto",
    ";;;; ",
    ";;;; Python:",
    ";;;;   PYTHON:             /usr/bin/python3",
    ";;;;   PYTHON_VERSION:     3.14.6",
    ";;;;   PKGPYTHONDIR:       /usr/lib/python3.14/site-packages/kea",
    ";;;; ",
    ";;;; Boost:                \"1_91\"",
    ";;;; ",
    ";;;; OpenSSL:              \"OpenSSL 3.6.3 9 Jun 2026\"",
    ";;;; ",
    ";;;; Log4cplus:            \"2.2.0\"",
    ";;;; ",
    ";;;; Flex:                 /usr/bin/flex",
    ";;;; Bison:                /usr/bin/bison",
    ";;;; ",
    ";;;; IO Event Handling:",
    ";;;;   select              yes",
    ";;;;   poll                yes",
    ";;;; ",
    ";;;; MySQL:                12.3.2",
    ";;;; ",
    ";;;; PostgreSQL:           18.4",
    ";;;; ",
    ";;;; Netconf:              no",
    ";;;; ",
    ";;;; Google Test:          1.17.0",
    ";;;; ",
    ";;;; Kerberos5 GSS-API:",
    ";;;;   VERSION:            1.22.2",
    ";;;;   VENDOR:             MIT",
    ";;;; ",
    ";;;; Developer:",
    ";;;;   Tests:              enabled",
    ";;;;   Fuzzing:            disabled",
    ";;;;   Valgrind:           no",
    ";;;;   AFL:                no",
    ""
};

}
}
